script_name("Teleporter")
script_author("Grinch_")
script_description("A teleporter menu for GTA San Andreas")
script_dependencies("imgui")
script_version("1.3")

--Keys
local keys =
{
	menu_key1		= 0xA2,  -- Left Control
	menu_key2 		= 0x54,  -- T
	quick_teleport1 = 0x58,  -- X
	quick_teleport2 = 0x59,  -- X
}

-- Script Dependencies
local imgui       = require 'imgui'

local teleporter =
{
	quick_teleport = imgui.ImBool(false),
	current_btn    = 1,
	current_coords = imgui.ImBool(false),
	auto_z		   = imgui.ImBool(false),
	show           = imgui.ImBool(false),
	show_coord     = imgui.ImBool(false),
	shotcut 	   = imgui.ImBool(false),
	coords  	   = imgui.ImBuffer(24),
	location_name  = imgui.ImBuffer(24),
	search_text    = imgui.ImBuffer(60),
	radio_button   = imgui.ImInt(0),
}

local resX, resY = getScreenResolution()


function LoadJson(filename)
    local full_path = getWorkingDirectory() .."/".. filename .. ".json"
    if doesFileExist(full_path) then
        local file = io.open(full_path, "r")
        local table = decodeJson(file:read("*a"))
        file:close()
        return table
    end
    return {}
end

function SaveJson(filename,table)
    local full_path = getWorkingDirectory() .. "/".. filename .. ".json"
    local file = assert(io.open(full_path, "w"))
    file:write(encodeJson(table))
    file:close()
end

local coordinates = LoadJson("Teleporter")

function Teleport(x, y, z,interior_id)
	if interior_id == nil then interior_id = 0 end

	if x == nil then
		_, x,y,z = getTargetBlipCoordinates()
		interior_id = 0
	end

	if interior_id == 0 then
		z = z + 3
	end

	if teleporter.auto_z.v then
		z = getGroundZFor3dCoord(x,y,z)
	end

	setCharInterior(PLAYER_PED,interior_id)
	setInteriorVisible(interior_id)
	clearExtraColours(true)
	requestCollision(x,y)
	activateInteriorPeds(true)
	setCharCoordinates(PLAYER_PED,x,y,z)
	loadScene(x,y,z)
end

function InformationTooltip(text)
	imgui.SameLine()
	imgui.TextColored(imgui.ImVec4(128,128,128,0.3),'(?)')
    if imgui.IsItemHovered() then
        if imgui.IsItemHovered() then
            imgui.BeginTooltip()
            imgui.Spacing()
            imgui.SetTooltip(text)
            imgui.Dummy(imgui.ImVec2(50,50))
            imgui.EndTooltip()
        end
    end
end

function ShowEntry(label, x, y, z,interior_id)
	if imgui.MenuItem(label, "", false, true) then
		Teleport(x, y, z,interior_id)
	end
	if imgui.IsItemClicked(1) then
		coordinates[label] = nil
		SaveJson("Teleporter",coordinates)
		coordinates = LoadJson("Teleporter")
		printHelpString("Entry ~r~removed")
	end
end


function GetSize(count,no_spacing)
  
    if count == 1 then no_spacing = true end

    if no_spacing == true then 
        x = imgui.GetWindowContentRegionWidth()/count
    else
        x = imgui.GetWindowContentRegionWidth()/count - imgui.StyleVar.ItemSpacing/(count*2)
    end

    y = (imgui.GetWindowHeight()/25)
    if y < 30 then y = 30 end

    return x,y
end

function CreateButtons(names,func)

    imgui.GetStyle().ItemSpacing = imgui.ImVec2(0.0,0.0)
	for i=1,#names,1 do
		
		if i == teleporter.current_btn then
			imgui.GetStyle().Colors[imgui.Col.Button] = imgui.ImVec4(0.06,  0.53,  0.98,  1.00)
		end

		if imgui.Button(names[i],imgui.ImVec2(GetSize(#names,true))) then
			teleporter.current_btn = i
		end

		imgui.GetStyle().Colors[imgui.Col.Button] = imgui.ImVec4(0.26,  0.59,  0.98,  0.40)

        if i%3 ~= 0 then
            imgui.SameLine(0.0,0.0)
		end

    end
    imgui.GetStyle().ItemSpacing = imgui.ImVec2(3.0,3.0)
	
	imgui.Dummy(imgui.ImVec2(0,10))
	if teleporter.show_coord.v then
		local x, y, z = getCharCoordinates(PLAYER_PED)
		imgui.Text(string.format("Player coordinates: %d %d %d ",  math.floor(x) ,  math.floor(y) ,  math.floor(z)))
		imgui.Dummy(imgui.ImVec2(0,10))
	end

    if imgui.BeginChild("") then
        
        func[teleporter.current_btn]()
        imgui.EndChild()
    end

end

function TeleportBtn()
	imgui.Columns(2, nil, false)
	imgui.Checkbox("Quick teleport",  teleporter.quick_teleport)
	InformationTooltip("Teleport to marker using (X + Y) key combinartion")
	imgui.Checkbox("Get ground coord",  teleporter.auto_z)
	InformationTooltip("Get Z ground coordinates of your coordinates")
	imgui.NextColumn()
	imgui.Checkbox("Show coordinates",  teleporter.show_coord)
	imgui.Checkbox("Insert coordinates",  teleporter.current_coords)
	InformationTooltip("Automatically inserts coordinates in the text field")
	imgui.Columns(1)

	if imgui.InputText("Coordinates", teleporter.coords) then
	end
	if teleporter.current_coords.v then
		local x, y, z = getCharCoordinates(PLAYER_PED)
		teleporter.coords.v = string.format("%d,  %d,  %d",  math.floor(x) ,  math.floor(y) ,  math.floor(z))
	end
	InformationTooltip("Enter XYZ coordinates.\nFormat : X, Y, Z")
	imgui.Dummy(imgui.ImVec2(0, 10))

	if imgui.Button("Teleport to coord", imgui.ImVec2(GetSize(2))) then
		x,y,z = teleporter.coords.v:match("([^,]+),([^,]+),([^,]+)")
		Teleport(x,y,z,nil)
	end
	imgui.SameLine()
	if imgui.Button("Teleport to marker", imgui.ImVec2(GetSize(2))) then
		Teleport()
	end
end

function SearchBtn()
	if imgui.InputText("Search", teleporter.search_text) then
	end
	imgui.SameLine()
	imgui.SameLine()
	InformationTooltip("Right click over any of these entries to remove them.")

	imgui.Spacing()
	imgui.Text("Found entries:(" .. teleporter.search_text.v .. ")")
	imgui.Separator()
	imgui.Spacing()
	if imgui.BeginChild("Entries") then
		for name, coord in pairs(coordinates) do
			local interior_id, x, y, z = coord:match("([^, ]+), ([^, ]+), ([^, ]+), ([^, ]+)")
			if teleporter.search_text.v == "" then
				ShowEntry(name, tonumber(x), tonumber(y), tonumber(z),interior_id)
			else
				if string.upper(name):find(string.upper(teleporter.search_text.v)) ~= nil  then
					ShowEntry(name, tonumber(x), tonumber(y), tonumber(z),interior_id)
				end
			end
		end
		imgui.EndChild()
	end
end

function CustomBtn()
	imgui.Spacing()
	imgui.Columns(1)
	if imgui.InputText("Location name", teleporter.location_name) then end
	if imgui.InputText("Coordinates", teleporter.coords) then end
	InformationTooltip("Enter XYZ coordinates.\nFormat : X,Y,Z")
	if teleporter.current_coords.v == true then
		local x,y,z = getCharCoordinates(PLAYER_PED)			
		teleporter.coords.v = string.format("%d, %d, %d", math.floor(x) , math.floor(y) , math.floor(z))
	end
	imgui.Spacing()
	if imgui.Button("Save location",imgui.ImVec2(GetSize(1))) then
		coordinates[teleporter.location_name.v] = string.format("%d, %s",getActiveInterior(), teleporter.coords.v)   
		SaveJson("Teleporter",coordinates)
		coordinates = LoadJson("Teleporter")
		printHelpString("Entry ~g~added")
	end
end

function imgui.OnDrawFrame()
	if  teleporter.show.v then

		imgui.SetNextWindowSize(imgui.ImVec2(resX/4, resY/1.2),  imgui.Cond.FirstUseEver)

		imgui.Begin(string.format( 'Teleporter v%s by Grinch_',script.this.version),teleporter.show,imgui.WindowFlags.NoCollapse)
		CreateButtons({"Teleport","Search","Custom"},{TeleportBtn,SearchBtn,CustomBtn})
        imgui.End()
    end
end

function main()
	ApplyStyle()

	while true do
		if isKeyDown(keys.menu_key1) and isKeyDown(keys.menu_key2) then
			while isKeyDown(keys.menu_key1) and isKeyDown(keys.menu_key2) do
				wait(0)
			end
			teleporter.show.v = not teleporter.show.v
		end

		if teleporter.quick_teleport.v == true then
			if isKeyDown(keys.quick_teleport1) and isKeyDown(keys.quick_teleport2) then
				while isKeyDown(keys.quick_teleport1) and isKeyDown(keys.quick_teleport2) do
					wait(0)
				end
				Teleport()
			end
		end

        imgui.Process = teleporter.show.v
        wait(0)
	end
end

function ApplyStyle()
    imgui.SwitchContext()
    local style = imgui.GetStyle()
    local colors = style.Colors
    local clr = imgui.Col
    local ImVec4 = imgui.ImVec4

    style.WindowRounding = 2.0
	style.WindowTitleAlign = imgui.ImVec2(0.5,  0.84)
	style.ChildWindowRounding = 2.0
	style.FrameRounding = 1.0
	style.ItemSpacing = imgui.ImVec2(3.0,  3.0)
	style.ScrollbarSize = 12.0
	style.ScrollbarRounding = 2.0
	style.GrabMinSize = 8.0
	style.GrabRounding = 1.0
    style.WindowPadding = imgui.ImVec2(10.0,  10.0)
	colors[clr.Text]                   = ImVec4(1.00,  1.00,  1.00,  1.00)
	colors[clr.TextDisabled]           = ImVec4(0.50,  0.50,  0.50,  1.00)
	colors[clr.WindowBg]               = ImVec4(0.06,  0.06,  0.06,  0.94)
	colors[clr.ChildWindowBg]          = ImVec4(1.00,  1.00,  1.00,  0.00)
	colors[clr.PopupBg]                = ImVec4(0.08,  0.08,  0.08,  0.94)
	colors[clr.ComboBg]                = colors[clr.PopupBg]
	colors[clr.Border]                 = ImVec4(0.43,  0.43,  0.50,  0.50)
	colors[clr.BorderShadow]           = ImVec4(0.00,  0.00,  0.00,  0.00)
	colors[clr.FrameBg]                = ImVec4(0.16,  0.29,  0.48,  0.54)
	colors[clr.FrameBgHovered]         = ImVec4(0.26,  0.59,  0.98,  0.40)
	colors[clr.FrameBgActive]          = ImVec4(0.26,  0.59,  0.98,  0.67)
	colors[clr.TitleBg]                = ImVec4(0.04,  0.04,  0.04,  1.00)
	colors[clr.TitleBgActive]          = ImVec4(0.16,  0.29,  0.48,  1.00)
	colors[clr.TitleBgCollapsed]       = ImVec4(0.00,  0.00,  0.00,  0.51)
	colors[clr.MenuBarBg]              = ImVec4(0.14,  0.14,  0.14,  1.00)
	colors[clr.ScrollbarBg]            = ImVec4(0.02,  0.02,  0.02,  0.53)
	colors[clr.ScrollbarGrab]          = ImVec4(0.31,  0.31,  0.31,  1.00)
	colors[clr.ScrollbarGrabHovered]   = ImVec4(0.41,  0.41,  0.41,  1.00)
	colors[clr.ScrollbarGrabActive]    = ImVec4(0.51,  0.51,  0.51,  1.00)
	colors[clr.CheckMark]              = ImVec4(0.26,  0.59,  0.98,  1.00)
	colors[clr.SliderGrab]             = ImVec4(0.24,  0.52,  0.88,  1.00)
	colors[clr.SliderGrabActive]       = ImVec4(0.26,  0.59,  0.98,  1.00)
	colors[clr.Button]                 = ImVec4(0.26,  0.59,  0.98,  0.40)
	colors[clr.ButtonHovered]          = ImVec4(0.26,  0.59,  0.98,  1.00)
	colors[clr.ButtonActive]           = ImVec4(0.06,  0.53,  0.98,  1.00)
	colors[clr.Header]                 = ImVec4(0.26,  0.59,  0.98,  0.31)
	colors[clr.HeaderHovered]          = ImVec4(0.26,  0.59,  0.98,  0.80)
	colors[clr.HeaderActive]           = ImVec4(0.26,  0.59,  0.98,  1.00)
	colors[clr.Separator]              = colors[clr.Border]
	colors[clr.SeparatorHovered]       = ImVec4(0.26,  0.59,  0.98,  0.78)
	colors[clr.SeparatorActive]        = ImVec4(0.26,  0.59,  0.98,  1.00)
	colors[clr.ResizeGrip]             = ImVec4(0.26,  0.59,  0.98,  0.25)
	colors[clr.ResizeGripHovered]      = ImVec4(0.26,  0.59,  0.98,  0.67)
	colors[clr.ResizeGripActive]       = ImVec4(0.26,  0.59,  0.98,  0.95)
	colors[clr.CloseButton]            = ImVec4(0.41,  0.41,  0.41,  0.50)
	colors[clr.CloseButtonHovered]     = ImVec4(0.98,  0.39,  0.36,  1.00)
	colors[clr.CloseButtonActive]      = ImVec4(0.98,  0.39,  0.36,  1.00)
	colors[clr.PlotLines]              = ImVec4(0.61,  0.61,  0.61,  1.00)
	colors[clr.PlotLinesHovered]       = ImVec4(1.00,  0.43,  0.35,  1.00)
	colors[clr.PlotHistogram]          = ImVec4(0.90,  0.70,  0.00,  1.00)
	colors[clr.PlotHistogramHovered]   = ImVec4(1.00,  0.60,  0.00,  1.00)
	colors[clr.TextSelectedBg]         = ImVec4(0.26,  0.59,  0.98,  0.35)
	colors[clr.ModalWindowDarkening]   = ImVec4(0.80,  0.80,  0.80,  0.35)
end

